#include <mathf.h>
#include <fxlib.h>
#include "draw.h"
#include "game.h"
#include "key.h"
#include "level.h"

int sin[360], cos[360];

void init_trigo()
{
	int i;
	for(i=0 ; i<360 ; i++)
	{
		sin[i] = 1000*sinf(i*3.141592654/180);
		cos[i] = 1000*cosf(i*3.141592654/180);
	}
}

void game(Level* level)
{
	char run=1, dead=0;
	int i, j, moving_counter=0;
	Camera camera;
	Character square, save_square;
	
/** Initialization **/
	camera.x = 8;
	camera.y = 8;
	for(i=1 ; i<level->height-1 ; i++)
	{
		for(j=1 ; j<level->width-1 ; j++)
		{
			if(level->map[i*level->width+j] == START)
			{
				square.x = 8*j;
				square.y = 8*i;
			}
		}
	}
	save_square = square;
	
/** Main loop **/
	while(run)
	{
		if(FastKeyDown(K_EXIT)) pause();
/** Movements **/
		moving_counter = (moving_counter+1)%MAX_SPEED;
		/** Square **/
		if(moving_counter == 0 && !dead)
		{
			if((FastKeyDown(K_UP) || FastKeyDown(K_8)) && level->map[square.y/8*level->width+(square.x+1)/8]<WALL_TOP && level->map[square.y/8*level->width+(square.x+6)/8]<WALL_TOP) square.y--;
			if((FastKeyDown(K_DOWN) || FastKeyDown(K_2) || FastKeyDown(K_5)) && level->map[(square.y+7)/8*level->width+(square.x+1)/8]<WALL_TOP && level->map[(square.y+7)/8*level->width+(square.x+6)/8]<WALL_TOP) square.y++;
			if((FastKeyDown(K_LEFT) || FastKeyDown(K_4)) && level->map[(square.y+1)/8*level->width+square.x/8]<WALL_TOP && level->map[(square.y+6)/8*level->width+square.x/8]<WALL_TOP) square.x--;
			if((FastKeyDown(K_RIGHT) || FastKeyDown(K_6)) && level->map[(square.y+1)/8*level->width+(square.x+7)/8]<WALL_TOP && level->map[(square.y+6)/8*level->width+(square.x+7)/8]<WALL_TOP) square.x++;
		}

		/** Balls on line **/
		for(i=0 ; i<level->nb_balls_on_line ; i++)
		{
			if(level->ball_on_line[i].dx)
			{
				if(moving_counter%(MAX_SPEED/level->ball_on_line[i].dx) == 0)
				{
					level->ball_on_line[i].x += sgn(level->ball_on_line[i].dx);
					if(level->map[(level->ball_on_line[i].y+1)/8*level->width + level->ball_on_line[i].x/8] >= WALL_TOP ||
					   level->map[(level->ball_on_line[i].y+6)/8*level->width + level->ball_on_line[i].x/8] >= WALL_TOP ||
					   level->map[(level->ball_on_line[i].y+1)/8*level->width + (level->ball_on_line[i].x+7)/8] >= WALL_TOP ||
					   level->map[(level->ball_on_line[i].y+6)/8*level->width + (level->ball_on_line[i].x+7)/8] >= WALL_TOP)
						level->ball_on_line[i].dx *= -1;
				}
			}
			if(level->ball_on_line[i].dy)
			{
				if(moving_counter%(MAX_SPEED/level->ball_on_line[i].dy) == 0)
				{
					level->ball_on_line[i].y += sgn(level->ball_on_line[i].dy);
					if(level->map[level->ball_on_line[i].y/8*level->width + (level->ball_on_line[i].x+1)/8] >= WALL_TOP ||
					   level->map[level->ball_on_line[i].y/8*level->width + (level->ball_on_line[i].x+6)/8] >= WALL_TOP ||
					   level->map[(level->ball_on_line[i].y+7)/8*level->width + (level->ball_on_line[i].x+1)/8] >= WALL_TOP ||
					   level->map[(level->ball_on_line[i].y+7)/8*level->width + (level->ball_on_line[i].x+6)/8] >= WALL_TOP)
						level->ball_on_line[i].dy *= -1;
				}
			}
		}
		
		/** Balls on path **/
		for(i=0 ; i<level->nb_balls_on_path ; i++)
		{
			if(level->ball_on_path[i].dx)
			{
				if(moving_counter%(MAX_SPEED/level->ball_on_path[i].dx) == 0)
					level->ball_on_path[i].x += sgn(level->ball_on_path[i].dx);
			}
			if(level->ball_on_path[i].dy)
			{
				if(moving_counter%(MAX_SPEED/level->ball_on_path[i].dy) == 0)
					level->ball_on_path[i].y += sgn(level->ball_on_path[i].dy);
			}
		}
		/** Balls on circle **/
		for(i=0 ; i<level->nb_balls_on_circle ; i++)
		{
			if(moving_counter%(MAX_SPEED/level->ball_on_circle[i].radius_speed)==0)
			{
				level->ball_on_circle[i].angle += sgn(level->ball_on_circle[i].radius_speed);
				level->ball_on_circle[i].angle %= 360;
				level->ball_on_circle[i].x = level->ball_on_circle[i].x_center + level->ball_on_circle[i].radius*cos[level->ball_on_circle[i].angle]/1000;
				level->ball_on_circle[i].y = level->ball_on_circle[i].y_center + level->ball_on_circle[i].radius*sin[level->ball_on_circle[i].angle]/1000;
			}
		}
		
/** Collisions **/
		/** Square & goal **/
		if(level->nb_catching_balls == 0 && level->map[(square.y+4)/8*level->width+(square.x+4)/8] == GOAL) run = 0;
		
		/** Square & catching balls **/
		if(level->map[(square.y+4)/8*level->width+(square.x+4)/8] == BALL)
		{
			level->nb_catching_balls--;
			level->map[(square.y+4)/8*level->width+(square.x+4)/8] = EMPTY2;
		}
		
		/** Square & checkpoint **/
		if(level->map[(square.y+4)/8*level->width+(square.x+4)/8] == CHECKPOINT)
		{
				memcpy(level->save_map, level->map, level->width*level->height);
				level->save_nb_catching_balls = level->nb_catching_balls;
				save_square = square;
		}

		if(!dead)
		{
			/** Square & balls on line **/
			for(i=0 ; i<level->nb_balls_on_line ; i++)
			{
				if(abs(level->ball_on_line[i].x-square.x)<5 && abs(level->ball_on_line[i].y-square.y)<5)
					dead = 1;
			}
		
			/** Square & balls on path **/
			for(i=0 ; i<level->nb_balls_on_path ; i++)
			{
				if(abs(level->ball_on_path[i].x-square.x)<5 && abs(level->ball_on_path[i].y-square.y)<5)
					dead = 1;
			}
		
			/** Square & balls on circle **/
			for(i=0 ; i<level->nb_balls_on_circle ; i++)
			{
				if(abs(level->ball_on_circle[i].x-square.x)<5 && abs(level->ball_on_circle[i].y-square.y)<5)
					dead = 1;
			}
		}

		/** Balls on path **/
		for(i=0 ; i<level->nb_balls_on_path ; i++)
		{
			int x, y, dx, dy;
			x = level->ball_on_path[i].x;
			y = level->ball_on_path[i].y;
			dx = level->ball_on_path[i].dx;
			dy = level->ball_on_path[i].dy;
			if(x%8==0 && y%8==0 && ((dx?moving_counter%(MAX_SPEED/dx)==0:0) || (dy?moving_counter%(MAX_SPEED/dy)==0:0)))
			{
				switch(level->ball_on_path[i].direction)
				{
					case LEFT:
						if(dx)
						{
							if(IsEmpty(level->map[(y/8-sgn(dx))*level->width+x/8]))
							{
								level->ball_on_path[i].dy = -dx;
								level->ball_on_path[i].dx = 0;
							} else if(IsWall(level->map[y/8*level->width+x/8+sgn(dx)]))
							{
								level->ball_on_path[i].dy = dx;
								level->ball_on_path[i].dx = 0;
							}
						} else {
							if(IsEmpty(level->map[y/8*level->width+x/8+sgn(dy)]))
							{
								level->ball_on_path[i].dx = dy;
								level->ball_on_path[i].dy = 0;
							} else if(IsWall(level->map[(y/8+sgn(dy))*level->width+x/8]))
							{
								level->ball_on_path[i].dx = -dy;
								level->ball_on_path[i].dy = 0;
							}
						}
						break;
					case RIGHT:
						if(dx)
						{
							if(IsEmpty(level->map[(y/8+sgn(dx))*level->width+x/8]))
							{
								level->ball_on_path[i].dy = dx;
								level->ball_on_path[i].dx = 0;
							} else if(IsWall(level->map[y/8*level->width+x/8+sgn(dx)]))
							{
								level->ball_on_path[i].dy = -dx;
								level->ball_on_path[i].dx = 0;
							}
						} else {
							if(IsEmpty(level->map[y/8*level->width+x/8-sgn(dy)]))
							{
								level->ball_on_path[i].dx = -level->ball_on_path[i].dy;
								level->ball_on_path[i].dy = 0;
							} else if(IsWall(level->map[(y/8+sgn(dy))*level->width+x/8]))
							{
								level->ball_on_path[i].dx = dy;
								level->ball_on_path[i].dy = 0;
							}
						}
						break;
					default:
						if(dx)
						{
							if(level->map[y/8*level->width+x/8+sgn(dx)]!=EMPTY)
							{
								if(level->map[(y/8-1)*level->width+x/8] == EMPTY) level->ball_on_path[i].dy = -abs(dx);
								else level->ball_on_path[i].dy = abs(dx);
								level->ball_on_path[i].dx = 0;
							}
						} else {
							if(level->map[(y/8+sgn(dy))*level->width+x/8]!=EMPTY)
							{
								if(level->map[y/8*level->width+x/8-1] == EMPTY) level->ball_on_path[i].dx = -abs(dy);
								else level->ball_on_path[i].dx = abs(dy);
								level->ball_on_path[i].dy = 0;
							}
						}
				}
			}
		}

		if(dead)
		{
			dead++;
			if(dead == 3*TIME_IMAGE_EXPLOSION+2)
			{
				memcpy(level->map, level->save_map, level->width*level->height);
				level->nb_catching_balls = level->save_nb_catching_balls;
				square = save_square;
				dead = 0;
			}
		}
		
/** Moving camera **/
		camera.x = square.x-60;
		camera.y = square.y-28;
		if(camera.x < 8) camera.x = 8;
		if(camera.x > level->width*8-136) camera.x = level->width*8-136;
		if(camera.y < 8) camera.y = 8;
		if(camera.y > level->height*8-72) camera.y = level->height*8-72;

/** Drawing **/
		Bdisp_AllClr_VRAM();
		/** Map **/
		for(i=0 ; i<9 ; i++)
		{
			for(j=0 ; j<17 ; j++)
				draw_bmp(level->map[(i+camera.y/8)*level->width+j+camera.x/8], 8*j-camera.x%8, 8*i-camera.y%8);
		}
		/** Moving balls on line **/
		for(i=0 ; i<level->nb_balls_on_line ; i++)
			draw_bmp(FILLED_BALL, level->ball_on_line[i].x-camera.x, level->ball_on_line[i].y-camera.y);
		/** Moving balls on path **/
		for(i=0 ; i<level->nb_balls_on_path ; i++)
			draw_bmp(FILLED_BALL, level->ball_on_path[i].x-camera.x, level->ball_on_path[i].y-camera.y);
		/** Moving balls on circle **/
		for(i=0 ; i<level->nb_balls_on_circle ; i++)
			draw_bmp(FILLED_BALL, level->ball_on_circle[i].x-camera.x, level->ball_on_circle[i].y-camera.y);
		/** Square **/
		if(dead) draw_bmp(EXPLOSION1+(dead-2)/TIME_IMAGE_EXPLOSION, square.x-camera.x, square.y-camera.y);
		else draw_bmp(SQUARE, square.x-camera.x, square.y-camera.y);
		Bdisp_PutDisp_DD();
/** Delay **/
		Sleep(8);
	}
	Sleep(300);
}

void pause()
{
	int i, j;
	char* vram = ML_vram_adress();
	char bmp[] = {255, 255, 255, 255, 255, 255, 255, 255, 195, 140, 152, 67, 201, 36, 147, 207, 195, 4, 152, 199, 207, 36, 158, 79, 207, 38, 48, 195, 255, 255, 255, 255, 255, 255, 255, 255};
	for(i=0 ; i<9 ; i++)
	{
		for(j=0 ; j<4 ; j++)
			vram[(i+28<<4)+j+6] = bmp[(i<<2)+j];
	}
	IsKeyDown(KEY_CTRL_EXE);
	while(i != KEY_CTRL_EXE) GetKey(&i);
}
